@@ -1,7 +1,7 @@
 ##########################################################################
 
 # MODSPEC.TCL, module specification procedures
-# Copyright (C) 2016-2023 Xavier Delaruelle
+# Copyright (C) 2016-2024 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -455,7 +455,7 @@ proc modVariantCmp {pvrlist modvrlist {missmean 0}} {
       # modulefile and mod is extra specifier defined on command line
    }
    foreach pvr $pvrlist {
-      set pvrarr([lindex $pvr 0]) [lindex $pvr 1]
+      set pvrarr([lindex $pvr 0]) [lindex $pvr 2]
    }
 
    # no match if a specified variant is not found among module variants (and
@@ -636,6 +636,8 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
    }
    # get alternative names if mod is loading(1) or loaded(2)
    set altlist [switch -- $ismodlo {
+      7 {getLoadedAltname $mod {alias}}
+      6 {getLoadedAltname $mod {sym autosym}}
       5 {getAvailListFromVersSpec $mod}
       4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
@@ -659,7 +661,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
       if {$test eq {eqstart}} {
          set ret [string equal -length [string length $pmod/] $pmod/ $mod/]
          # apply comparison to alternative names if any and no match for mod
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string equal -length [string length $pmod/]\
                   $pmod/ $alt/]]} {
@@ -678,7 +680,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
          ##nagelfar ignore Non static subcommand
          set ret [string $test $pmod $mod]
          # apply comparison to alternative names if any and no match for mod
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                ##nagelfar ignore Non static subcommand
                if {[set ret [string $test $pmod $alt]]} {
@@ -717,7 +719,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
          set ret 0
       }
       # apply comparison to alternative names if any and no match for mod
-      if {!$ret && [llength $altlist] > 0} {
+      if {!$ret && [llength $altlist]} {
          foreach alt $altlist {
             if {($pmodnamere ne {} && $test eq {match} && [regexp\
                (^$pmodnamere)/ $alt/ rematch pmodname]) || [string equal\
@@ -733,7 +735,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
       }
    }
    # check if variant specified matches those of selected loaded/ing module
-   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist] > 0} {
+   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist]} {
       if {$modvrlist eq {0}} {
          set modvrlist [getVariantList $mod]
       }
@@ -762,6 +764,8 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      7 {getLoadedAltname $mod {alias}}
+      6 {getLoadedAltname $mod {sym autosym}}
       5 {getAvailListFromVersSpec $mod}
       4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
@@ -783,7 +787,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       if {$test eq {eqstart}} {
          set ret [string equal -nocase -length [string length $pmod/] $pmod/\
             $mod/]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string equal -nocase -length [string length\
                   $pmod/] $pmod/ $alt/]]} {
@@ -801,7 +805,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          }
          ##nagelfar ignore Non static subcommand
          set ret [string $test -nocase $pmod $mod]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                ##nagelfar ignore Non static subcommand
                if {[set ret [string $test -nocase $pmod $alt]]} {
@@ -839,7 +843,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       } else {
          set ret 0
       }
-      if {!$ret && [llength $altlist] > 0} {
+      if {!$ret && [llength $altlist]} {
          foreach alt $altlist {
             if {($pmodnamere ne {} && $test eq {match} && [regexp -nocase\
                (^$pmodnamere)/ $alt/ rematch pmodname]) || [string equal\
@@ -854,7 +858,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          }
       }
    }
-   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist] > 0} {
+   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist]} {
       if {$modvrlist eq {0}} {
          set modvrlist [getVariantList $mod]
       }
@@ -881,6 +885,8 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      7 {getLoadedAltname $mod {alias}}
+      6 {getLoadedAltname $mod {sym autosym}}
       5 {getAvailListFromVersSpec $mod}
       4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
@@ -901,7 +907,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       }
       if {$test eq {eqstart}} {
          set ret [string equal -length [string length $pmod/] $pmod/ $mod/]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string equal -length [string length $pmod/]\
                   $pmod/ $alt/]]} {
@@ -920,7 +926,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          }
          ##nagelfar ignore Non static subcommand
          set ret [string $test $pmod $mod]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                ##nagelfar ignore Non static subcommand
                if {[set ret [string $test $pmod $alt]]} {
@@ -937,7 +943,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
             set pmodextdfl $pmodescglob.*
          }
          set ret [string match $pmodextdfl $mod]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string match $pmodextdfl $alt]]} {
                   break
@@ -974,7 +980,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       } else {
          set ret 0
       }
-      if {!$ret && [llength $altlist] > 0} {
+      if {!$ret && [llength $altlist]} {
          foreach alt $altlist {
             if {($pmodnamere ne {} && $test eq {match} && [regexp\
                (^$pmodnamere)/ $alt/ rematch pmodname]) || [string equal\
@@ -989,7 +995,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          }
       }
    }
-   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist] > 0} {
+   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist]} {
       if {$modvrlist eq {0}} {
          set modvrlist [getVariantList $mod]
       }
@@ -1016,6 +1022,8 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
       set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      7 {getLoadedAltname $mod {alias}}
+      6 {getLoadedAltname $mod {sym autosym}}
       5 {getAvailListFromVersSpec $mod}
       4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
@@ -1037,7 +1045,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
       if {$test eq {eqstart}} {
          set ret [string equal -nocase -length [string length $pmod/] $pmod/\
             $mod/]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string equal -nocase -length [string length\
                   $pmod/] $pmod/ $alt/]]} {
@@ -1056,7 +1064,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
          }
          ##nagelfar ignore Non static subcommand
          set ret [string $test -nocase $pmod $mod]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                ##nagelfar ignore Non static subcommand
                if {[set ret [string $test -nocase $pmod $alt]]} {
@@ -1073,7 +1081,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
             set pmodextdfl $pmodescglob.*
          }
          set ret [string match -nocase $pmodextdfl $mod]
-         if {!$ret && [llength $altlist] > 0} {
+         if {!$ret && [llength $altlist]} {
             foreach alt $altlist {
                if {[set ret [string match -nocase $pmodextdfl $alt]]} {
                   break
@@ -1110,7 +1118,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
       } else {
          set ret 0
       }
-      if {!$ret && [llength $altlist] > 0} {
+      if {!$ret && [llength $altlist]} {
          foreach alt $altlist {
             if {($pmodnamere ne {} && $test eq {match} && [regexp -nocase\
                (^$pmodnamere)/ $alt/ rematch pmodname]) || [string equal\
@@ -1125,7 +1133,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
          }
       }
    }
-   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist] > 0} {
+   if {$ret && $vrcmp && $ismodlo ni {3 5} && [llength $pvrlist]} {
       if {$modvrlist eq {0}} {
          set modvrlist [getVariantList $mod]
       }
@@ -1294,7 +1302,7 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
    set xtelt_valid_list [list always-load append-path chdir complete conflict\
       depends-on envvar family incompat load load-any prepend-path prereq\
       prereq-all prereq-any pushenv remove-path require set-alias\
-      set-function setenv switch switch-on switch-off try-load uncomplete\
+      set-function setenv switch switch-on switch-off tag try-load uncomplete\
       unload unset-alias unset-function unsetenv variant]
    set xtelt_modspec_list [list always-load conflict depends-on incompat load\
       load-any prereq prereq-all prereq-any require switch switch-on\
@@ -1397,38 +1405,64 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
                # extract extra specifier spec
                set xtsepidx [string first : $curarg]
                set xtelt [string range $curarg 0 $xtsepidx-1]
-               set xtname [string range $curarg $xtsepidx+1 end]
+               set xtnamelist [split [string range $curarg $xtsepidx+1 end] ,]
 
                # check no other : character is found in argument or element
                # and name are not an empty string
-               if {[string length $xtelt] == 0 || [string length $xtname] ==\
-                  0 || ([string last : $curarg] != $xtsepidx && $xtelt ni\
-                  $xtelt_modspec_list)} {
+               if {![string length $xtelt] || ![llength $xtnamelist] || {} in\
+                  $xtnamelist || ([string last : $curarg] != $xtsepidx &&\
+                  $xtelt ni $xtelt_modspec_list)} {
                   knerror "Invalid extra specification '$arg'"
                }
                if {$xtelt ni $xtelt_valid_list} {
                   knerror "Invalid extra specifier '$xtelt'\nValid extra\
                      specifiers are: $xtelt_valid_list"
                }
+               set spec_xt [list $xtelt]
                # parse and resolve module spec set as extra specifier value
                if {$xtelt in $xtelt_modspec_list} {
-                  lassign [parseModuleSpecification 0 0 0 1 {*}$xtname] xtname
+                  foreach xtname $xtnamelist {
+                     lassign [parseModuleSpecification 0 0 0 1 {*}$xtname]\
+                        xtname
+                     lappend spec_xt $xtname
+                  }
+               } else {
+                  # For tag extra specifier, resolve each name to get
+                  # corresponding tag name if tag abbreviation set as name
+                  if {$xtelt eq {tag}} {
+                     foreach xtname $xtnamelist {
+                        lappend spec_xt $xtname
+                        if {[set tag [getTagFromAbbrev $xtname]] ne {}} {
+                           lappend spec_xt $tag
+                        }
+                     }
+                  } else {
+                     lappend spec_xt {*}$xtnamelist
+                  }
                }
-               # save extra specifier element and name value, same element can
-               # appear multiple time (means AND operator)
-               lappend xtlist [list $xtelt $xtname]
+               # save extra specifier element and name value, multiple values
+               # may be set (means OR operator), same element can appear
+               # multiple time (means AND operator)
+               lappend xtlist $spec_xt
             }
             *=* {
                # extract valued-variant spec
                set vrsepidx [string first = $curarg]
                set vrname [string range $curarg 0 $vrsepidx-1]
-               set vrvalue [string range $curarg $vrsepidx+1 end]
+               set vrvaluelist [split [string range $curarg $vrsepidx+1 end]\
+                  ,]
+               # value is one empty string
+               if {![llength $vrvaluelist]} {
+                  lappend vrvaluelist {}
+               }
 
                if {$vrname eq {}} {
                   knerror "No variant name defined in argument '$curarg'"
                }
-               # check no other = character is found in argument
-               if {[string last = $curarg] != $vrsepidx} {
+               # check no other = character is found in argument and that only
+               # one value is set unless if extra specifier search is enabled
+               if {[string last = $curarg] != $vrsepidx || (!$xtspec &&\
+                  [llength $vrvaluelist] > 1)} {
                   knerror "Invalid variant specification '$arg'"
                }
                # replace previous value for variant if already set unless if
@@ -1440,14 +1474,23 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
                   incr vridx
                }
                # translate boolean vrvalue in canonical boolean
-               if {!$vrisbool && [string is boolean -strict $vrvalue] &&\
-                  ![string is integer -strict $vrvalue]} {
+               if {!$vrisbool} {
                   set vrisbool 1
-                  set vrvalue [string is true -strict $vrvalue]
+                  for {set i 0} {$i < [llength $vrvaluelist]} {incr i 1} {
+                     set vrvalue [lindex $vrvaluelist $i]
+                     if {[string is boolean -strict $vrvalue] && ![string is\
+                        integer -strict $vrvalue]} {
+                        lset vrvaluelist $i [string is true -strict $vrvalue]
+                     } else {
+                        # consider variant not a boolean as soon as one value
+                        # set is not a boolean
+                        set vrisbool 0
+                     }
+                  }
                }
                # save variant name and value
                set vrnamearr($vrname) $vridx
-               lappend vrlist [list $vrname $vrvalue $vrisbool]
+               lappend vrlist [list $vrname $vrisbool {*}$vrvaluelist]
             }
             default {
                # save previous mod version spec and transformed arg if any
@@ -1722,6 +1765,23 @@ proc getExtraListFromVersSpec {modarg} {
    return $extralist
 }
 
+# translate module name version spec to return list of tag specifier and list
+# of other extra specifier
+proc getSplitExtraListFromVersSpec {modarg} {
+   set taglist {}
+   set otherlist {}
+   if {[info exists ::g_moduleVersSpec($modarg)]} {
+      foreach spec_xt [lindex $::g_moduleVersSpec($modarg) 10] {
+         if {[lindex $spec_xt 0] eq {tag}} {
+            lappend taglist $spec_xt
+         } else {
+            lappend otherlist $spec_xt
+         }
+      }
+   }
+   return [list $taglist $otherlist]
+}
+
 # get available modules matching module specification
 proc getAvailListFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
@@ -1735,7 +1795,7 @@ proc getAvailListFromVersSpec {modarg} {
 # is mod spec containing variant but no module name and version
 proc isSpecWildWithVariant {mod} {
    return [expr {[getModuleNameAndVersFromVersSpec $mod] eq {*} && [llength\
-      [getVariantListFromVersSpec $mod]] > 0}]
+      [getVariantListFromVersSpec $mod]]}]
 }
 
 # ;;; Local Variables: ***
